/* Copyright 2019 The TensorFlow Authors. All Rights Reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
==============================================================================*/

#include "tensorflow/lite/experimental/micro/examples/cds_position/constants.h"
#include "tensorflow/lite/experimental/micro/examples/cds_position/output_handler.h"
#include "tensorflow/lite/experimental/micro/examples/cds_position/cds_position_model_data.h"
#include "tensorflow/lite/experimental/micro/kernels/all_ops_resolver.h"
#include "tensorflow/lite/experimental/micro/micro_error_reporter.h"
#include "tensorflow/lite/experimental/micro/micro_interpreter.h"
#include "tensorflow/lite/schema/schema_generated.h"
#include "tensorflow/lite/version.h"

int tflite_micro_main(int argc, char* argv[]) {
  // Set up logging
  tflite::MicroErrorReporter micro_error_reporter;
  tflite::ErrorReporter* error_reporter = &micro_error_reporter;

  // Map the model into a usable data structure. This doesn't involve any
  // copying or parsing, it's a very lightweight operation.
  const tflite::Model* model = ::tflite::GetModel(cds_model_quantized_tflite);
  if (model->version() != TFLITE_SCHEMA_VERSION) {
    error_reporter->Report(
        "Model provided is schema version %d not equal "
        "to supported version %d.",
        model->version(), TFLITE_SCHEMA_VERSION);
    return 1;
  }

  // This pulls in all the operation implementations we need
  tflite::ops::micro::AllOpsResolver resolver;

  // Create an area of memory to use for input, output, and intermediate arrays.
  // Finding the minimum value for your model may require some trial and error.
  const int tensor_arena_size = 2 * 1024;
  uint8_t tensor_arena[tensor_arena_size];

  // Build an interpreter to run the model with
  tflite::MicroInterpreter interpreter(model, resolver, tensor_arena,
                                       tensor_arena_size, error_reporter);

  // Allocate memory from the tensor_arena for the model's tensors
  interpreter.AllocateTensors();

  // Obtain pointers to the model's input and output tensors
  TfLiteTensor* input = interpreter.input(0);
  TfLiteTensor* output = interpreter.output(0);
  error_reporter->Report("model input: %d, %d, %d", input->dims->size, input->type);
  error_reporter->Report("output input: %d, %d", output->dims->size, output->type);

  // Keep track of how many inferences we have performed
  int inference_count = 0;

  // Loop indefinitely
  while (true) {

    // Place our calculated x value in the model's input tensor
    int rightBottom = getRightBottom();
    int rightTop = getRightTop();
    int leftTop = getLeftTop();
    int leftBottom = getLeftBottom();
    input->data.f[0] = rightBottom;
    input->data.f[1] = rightTop;
    input->data.f[2] = leftTop;
    input->data.f[3] = leftBottom;

    // Run inference, and report any error
    TfLiteStatus invoke_status = interpreter.Invoke();
    if (invoke_status != kTfLiteOk) {
      error_reporter->Report("Invoke failed!!!");
      continue;
    }

    // Read the predicted y value from the model's output tensor
    float y_label1 = output->data.f[0];
    float y_label2 = output->data.f[1];
    float y_label3 = output->data.f[2];
    float y_label4 = output->data.f[3];
    float y_label5 = output->data.f[4];

    // Output the results. A custom HandleOutput function can be implemented
    // for each supported hardware target.
    HandleOutput(error_reporter, y_label1, y_label2, y_label3, y_label4, y_label5);
  }
}
